@@ -24,7 +24,6 @@ use byteorder::ReadBytesExt;
 use byteorder::WriteBytesExt;
 use std::ffi::CString;
 use std::io;
-use std::mem;
 use std::time;
 
 use checksum;
@@ -145,10 +144,15 @@ impl HeaderBuilder {
     /// assert_eq!(header.comment(), None);
     /// ```
     pub fn new() -> Self {
+        // wasm-unknown-unknown does not implement the time module
+        #[cfg(not(target_arch = "wasm32"))]
         let modification_time = time::UNIX_EPOCH
             .elapsed()
             .map(|d| d.as_secs() as u32)
             .unwrap_or(0);
+        #[cfg(target_arch = "wasm32")]
+        let modification_time = 0;
+
         let header = Header {
             modification_time,
             compression_level: CompressionLevel::Unknown,
@@ -221,9 +225,10 @@ impl HeaderBuilder {
     ///
     /// # Examples
     /// ```
-    /// use libflate::gzip::{HeaderBuilder, ExtraField};
+    /// use libflate::gzip::{HeaderBuilder, ExtraField, ExtraSubField};
     ///
-    /// let extra = ExtraField{id: [0, 1], data: vec![2, 3, 4]};
+    /// let subfield = ExtraSubField{id: [0, 1], data: vec![2, 3, 4]};
+    /// let extra = ExtraField{subfields: vec![subfield]};
     /// let header = HeaderBuilder::new().extra_field(extra.clone()).finish();
     /// assert_eq!(header.extra_field(), Some(&extra));
     /// ```
@@ -334,10 +339,10 @@ impl Header {
             (F_NAME, self.filename.is_some()),
             (F_COMMENT, self.comment.is_some()),
         ]
-            .iter()
-            .filter(|e| e.1)
-            .map(|e| e.0)
-            .sum()
+        .iter()
+        .filter(|e| e.1)
+        .map(|e| e.0)
+        .sum()
     }
     fn crc16(&self) -> u16 {
         let mut crc = checksum::Crc32::new();
@@ -345,7 +350,8 @@ impl Header {
         Header {
             is_verified: false,
             ..self.clone()
-        }.write_to(&mut buf)
+        }
+        .write_to(&mut buf)
         .unwrap();
         crc.update(&buf);
         crc.value() as u16
@@ -447,35 +453,63 @@ where
 /// Extra field of a GZIP header.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct ExtraField {
-    /// ID of the extra field.
-    pub id: [u8; 2],
-
     /// Data of the extra field.
-    pub data: Vec<u8>,
+    pub subfields: Vec<ExtraSubField>,
 }
 impl ExtraField {
     fn read_from<R>(mut reader: R) -> io::Result<Self>
     where
         R: io::Read,
     {
-        let mut extra = ExtraField {
+        let mut subfields = Vec::new();
+        let data_size = reader.read_u16::<LittleEndian>()? as usize;
+        let mut reader = reader.take(data_size as u64);
+        while reader.limit() > 0 {
+            subfields.push(ExtraSubField::read_from(&mut reader)?);
+        }
+        Ok(ExtraField { subfields })
+    }
+    fn write_to<W>(&self, mut writer: W) -> io::Result<()>
+    where
+        W: io::Write,
+    {
+        let len = self.subfields.iter().map(|f| f.write_len()).sum::<usize>();
+        if len > 0xFFFF {
+            return Err(invalid_data_error!("extra field too long: {}", len));
+        }
+        writer.write_u16::<LittleEndian>(len as u16)?;
+        for f in &self.subfields {
+            f.write_to(&mut writer)?;
+        }
+        Ok(())
+    }
+}
+
+/// A sub field in the extra field of a GZIP header.
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+pub struct ExtraSubField {
+    /// ID of the field.
+    pub id: [u8; 2],
+
+    /// Data of the field.
+    pub data: Vec<u8>,
+}
+impl ExtraSubField {
+    fn read_from<R>(mut reader: R) -> io::Result<Self>
+    where
+        R: io::Read,
+    {
+        let mut field = ExtraSubField {
             id: [0; 2],
             data: Vec::new(),
         };
+
+        reader.read_exact(&mut field.id)?;
         let data_size = reader.read_u16::<LittleEndian>()? as usize;
-        if data_size < 2 {
-            return Err(invalid_data_error!(
-                "extra field is too short: {}",
-                data_size
-            ));
-        }
+        field.data.resize(data_size, 0);
+        reader.read_exact(&mut field.data)?;
 
-        reader.read_exact(&mut extra.id)?;
-
-        extra.data.resize(data_size - 2, 0);
-        reader.read_exact(&mut extra.data)?;
-
-        Ok(extra)
+        Ok(field)
     }
     fn write_to<W>(&self, mut writer: W) -> io::Result<()>
     where
@@ -486,6 +520,9 @@ impl ExtraField {
         writer.write_all(&self.data)?;
         Ok(())
     }
+    fn write_len(&self) -> usize {
+        4 + self.data.len()
+    }
 }
 
 /// OS type.
@@ -1080,33 +1117,43 @@ where
             Err(_) => return Ok(0),
             Ok(ref mut decoder) => decoder.read(buf)?,
         };
+        // take_mut closure must have the type it borrows as return type,
+        // so we put the function return result to this variable instead.
+        // If function logic is correct, these initial values will never be returned.
+        let mut result: io::Result<usize> = Err(io::Error::new(
+            io::ErrorKind::Other,
+            "If you see this error, please report a bug in libflate",
+        ));
         if read_size == 0 {
-            let mut reader = mem::replace(&mut self.decoder, Err(unsafe { mem::uninitialized() }))
-                .ok()
-                .take()
-                .expect("Never fails")
-                .into_inner();
-            match Header::read_from(&mut reader) {
-                Err(e) => {
-                    mem::forget(mem::replace(&mut self.decoder, Err(reader)));
-                    if e.kind() == io::ErrorKind::UnexpectedEof {
-                        Ok(0)
-                    } else {
-                        Err(e)
+            take_mut::take(self, |mut owned_self| {
+                let mut reader = owned_self
+                    .decoder
+                    .ok()
+                    .take()
+                    .expect("Never fails")
+                    .into_inner();
+                match Header::read_from(&mut reader) {
+                    Err(e) => {
+                        if e.kind() == io::ErrorKind::UnexpectedEof {
+                            result = Ok(0);
+                        } else {
+                            result = Err(e);
+                        }
+                        owned_self.decoder = Err(reader);
+                        owned_self
+                    }
+                    Ok(header) => {
+                        owned_self.header = header.clone();
+                        owned_self.decoder = Ok(Decoder::with_header(reader, header));
+                        result = owned_self.read(buf);
+                        owned_self
                     }
                 }
-                Ok(header) => {
-                    self.header = header.clone();
-                    mem::forget(mem::replace(
-                        &mut self.decoder,
-                        Ok(Decoder::with_header(reader, header)),
-                    ));
-                    self.read(buf)
-                }
-            }
+            })
         } else {
-            Ok(read_size)
+            result = Ok(read_size);
         }
+        result
     }
 }
 
@@ -1114,7 +1161,7 @@ where
 mod test {
     use super::*;
     use finish::AutoFinish;
-    use std::io;
+    use std::io::{self, Write};
 
     fn decode(buf: &[u8]) -> io::Result<Vec<u8>> {
         let mut decoder = Decoder::new(buf).unwrap();
@@ -1188,4 +1235,47 @@ mod test {
         let data = b"\x1F\x8B\x08\xC1\x91\x28\x71\xDC\xF2\x2D\x34\x35\x31\x35\x34\x30\x70\x6E\x60\x35\x31\x32\x32\x33\x32\x33\x37\x32\x36\x38\xDD\x1C\xE5\x2A\xDD\xDD\xDD\x22\xDD\xDD\xDD\xDC\x88\x13\xC9\x40\x60\xA7";
         assert!(decode(&data[..]).is_err());
     }
+
+    #[test]
+    fn extra_field() {
+        let f = ExtraField {
+            subfields: vec![ExtraSubField {
+                id: [0, 0x42],
+                data: "abc".into(),
+            }],
+        };
+
+        let mut buf = Vec::new();
+        f.write_to(&mut buf).unwrap();
+
+        assert_eq!(ExtraField::read_from(&buf[..]).unwrap(), f);
+    }
+
+    #[test]
+    fn encode_with_extra_field() {
+        let mut buf = Vec::new();
+        let extra_field = ExtraField {
+            subfields: vec![ExtraSubField {
+                id: [0, 0x42],
+                data: "abc".into(),
+            }],
+        };
+        {
+            // encode
+            let header = HeaderBuilder::new()
+                .extra_field(extra_field.clone())
+                .finish();
+
+            let ops = EncodeOptions::new().header(header);
+            let mut encoder = Encoder::with_options(&mut buf, ops).unwrap();
+            write!(encoder, "hello world").unwrap();
+            encoder.finish().as_result().unwrap();
+        }
+        {
+            // decode
+            let mut decoder = Decoder::new(&buf[..]).unwrap();
+            io::copy(&mut decoder, &mut io::sink()).unwrap();
+            assert_eq!(decoder.header().extra_field(), Some(&extra_field));
+        }
+    }
 }
