@@ -1,9 +1,10 @@
 //! A Rust implementation of DEFLATE algorithm and related formats (ZLIB, GZIP).
 #![warn(missing_docs)]
-#![cfg_attr(feature = "cargo-clippy", allow(inline_always))]
 extern crate adler32;
 extern crate byteorder;
 extern crate crc32fast;
+extern crate rle_decode_fast;
+extern crate take_mut;
 
 pub use finish::Finish;
 
